==> GPS_datacollector_dagur-0.1.0.crate: GPS_datacollector_dagur-0.1.0/src/lib.rs <==
//! GPS data fetcher for mechatronics
//! ## EXAMPLES
//! ```
//!     let mut gps = GpsInfo::new();
//!     gps.get_data();
//! ```
//! ## Important info:
//! ```
//! Reads only GPGGA
//! ```



use rpi_embedded::uart::{Parity, Uart};
use std::thread;
use std::time::{Duration};


//Sinple test chching if GPS is GPGGA
/*
#[cfg(test)]
mod tests{
   use super::*;
   
   #[test]
    fn isitgpgga() {
        let mut gps1 = GpsInfo::new();
        let input = "$GPGGA";
        assert_eq!(true, gps1.isgpgga(input));
        
    }
}
*/
enum BaudRate {
    A,
    B,
}
 
impl BaudRate{
    fn value(&self) -> u32{
        match *self {
            BaudRate::A => 9600,
            BaudRate::B => 115200,
        }
    }
}

struct GpsInfo{
    identifier: String,
    time: f64,
    latitude: f64,
    longitude: f64,
    ns: String,
    ew: String,
    sat_num: u16,
    altitude: f64,
    uart_serial: Uart,
    uart_baudrate: BaudRate,

}


impl GpsInfo {
    fn new() -> GpsInfo{

        println!("starting UART");

        let baudrate = BaudRate::A;

        let mut out = GpsInfo {
            identifier: "".to_string(),
            time: 0.0,
            latitude: 0.0,
            longitude: 0.0,
            ns: "".to_string(),
            ew: "".to_string(),
            sat_num: 0,
            altitude: 0.0,            
            uart_serial: Uart::new(baudrate.value(), Parity::None, 8, 1).expect("Failed to init uart serial"),
            uart_baudrate: BaudRate::A,
        };

        println!("starting UART");
        out.uart_serial.set_read_mode(1, Duration::default()).expect("Failed to set uart read mode");

        out

    }

    fn get_data(&mut self){

        loop{
        
            let res = self.uart_serial.read_line().expect("Fail in uart read");
            let mut v: Vec<&str> = res.split(',').collect();
            //println!("{}", v[0]);
            thread::sleep(Duration::from_millis(100));

            if self.isgpgga(v[0]){
                self.turn_vales(&mut v);
            }


        }
    }

    fn isgpgga(&mut self, data: &str) -> bool{
        let gppa: bool;
        if data.trim() == "$GPGGA"{
            gppa = true;
        }
        else{
            gppa = false;
        }
        return gppa;

    }


    //fn turn_vales(&mut self, data: &mut [&str]){
    fn turn_vales(&mut self, data: &mut Vec<&str>){
        self.identifier = data[0].to_string();
        self.time = data[1].parse::<f64>().unwrap();
        self.latitude = data[2].parse::<f64>().unwrap();
        self.longitude = data[4].parse::<f64>().unwrap();
        self.ns = data[3].to_string();
        self.ew = data[5].to_string();
        self.sat_num = data[7].parse::<u16>().unwrap();
        self.altitude = data[9].parse::<f64>().unwrap();
        println!("Ident:{} Time:{} Lat:{} Long:{} Alt:{} satnum:{} N/S:{} EW/:{}", self.identifier, self.time,self.latitude, self.longitude, self.altitude, self.sat_num, self.ns, self.ew);

    }
}


fn main() {
    let mut gps = GpsInfo::new();
    gps.get_data();
}

==> GPS_datacollector_dagur-0.1.1.crate: GPS_datacollector_dagur-0.1.1/src/lib.rs <==
//! GPS data fetcher for mechatronics
//! ## EXAMPLES
//! ```
//!     let mut gps = GpsInfo::new();
//!     gps.get_data();
//! ```
//! ## Important info:
//! ```
//! Reads only GPGGA
//! ```



use rpi_embedded::uart::{Parity, Uart};
use std::thread;
use std::time::{Duration};


//Simple test checking if GPS is GPGGA

#[cfg(test)]
mod tests{
   use super::*;
   
   #[test]
    fn isitgpgga() {
        let mut gps1 = GpsInfo::new();
        let input = "$GPGGA";
        assert_eq!(true, gps1.isgpgga(input));
        
    }
}

enum BaudRate {
    A,
    B,
}
 
impl BaudRate{
    fn value(&self) -> u32{
        match *self {
            BaudRate::A => 9600,
            BaudRate::B => 115200,
        }
    }
}

struct GpsInfo{
    identifier: String,
    time: f64,
    latitude: f64,
    longitude: f64,
    ns: String,
    ew: String,
    sat_num: u16,
    altitude: f64,
    uart_serial: Uart,
    uart_baudrate: BaudRate,

}


impl GpsInfo {
    fn new() -> GpsInfo{

        println!("starting UART");

        let baudrate = BaudRate::A;

        let mut out = GpsInfo {
            identifier: "".to_string(),
            time: 0.0,
            latitude: 0.0,
            longitude: 0.0,
            ns: "".to_string(),
            ew: "".to_string(),
            sat_num: 0,
            altitude: 0.0,            
            uart_serial: Uart::new(baudrate.value(), Parity::None, 8, 1).expect("Failed to init uart serial"),
            uart_baudrate: BaudRate::A,
        };

        println!("starting UART");
        out.uart_serial.set_read_mode(1, Duration::default()).expect("Failed to set uart read mode");

        out

    }

    fn get_data(&mut self){

        loop{
        
            let res = self.uart_serial.read_line().expect("Fail in uart read");
            let mut v: Vec<&str> = res.split(',').collect();
            //println!("{}", v[0]);
            thread::sleep(Duration::from_millis(100));

            if self.isgpgga(v[0]){
                self.turn_vales(&mut v);
            }


        }
    }

    fn isgpgga(&mut self, data: &str) -> bool{
        let gppa: bool;
        if data.trim() == "$GPGGA"{
            gppa = true;
        }
        else{
            gppa = false;
        }
        return gppa;

    }


    //fn turn_vales(&mut self, data: &mut [&str]){
    fn turn_vales(&mut self, data: &mut Vec<&str>){
        self.identifier = data[0].to_string();
        self.time = data[1].parse::<f64>().unwrap();
        self.latitude = data[2].parse::<f64>().unwrap();
        self.longitude = data[4].parse::<f64>().unwrap();
        self.ns = data[3].to_string();
        self.ew = data[5].to_string();
        self.sat_num = data[7].parse::<u16>().unwrap();
        self.altitude = data[9].parse::<f64>().unwrap();
        println!("Ident:{} Time:{} Lat:{} Long:{} Alt:{} satnum:{} N/S:{} EW/:{}", self.identifier, self.time,self.latitude, self.longitude, self.altitude, self.sat_num, self.ns, self.ew);

    }
}


fn main() {
    let mut gps = GpsInfo::new();
    gps.get_data();
}
